c***********************************************************************
c
c       SUBROUTINE COND
c
c       SUBROUTINE COND CALCULATES IN SITU THERMAL CONDUCTIVITY
c       USING THE DECAY OF A HEAT PULSE (PULSE PROBE METHOD AFTER
c       LISTER (1979)
c
c       DETAILS OF THE USED ALGORITHM ARE EXPLAINED IN :
c
c       HEINER VILLINGER AND E.E.DAVIS (1987)
c       A NEW REDUCTION ALGORITHM FOR MARINE HEAT-FLOW MEASUREMENTS
c       JOURNAL OF GEOPHYSICAL RESEARCH, IN PRESS
c
c       VARIABLES
c       ---------
c       I : SENSOR NUMBER
c       NUMB : NUMBER OF DATA POINTS
c       TIME : TIME
c       TEMP : TEMPERATURE
c       AXE : INFINITE TIME TEMPERATURE
c       EST : 95% ERROR ESTIMATE FOR INFINITE TIME TEMPERATURES
c       NUSED : NUMBER OF POINTS USED
c       FTAU : F(ALPHA,TAU) VALUES FOR THE DATA POINTS
c       TEM : CORRECTED TEMPERATURES
c       K : CALCULATED THERMAL CONDUCTIVITY
c       RMS : STANDART DEVIATION OF THE THERMAL CONDUCTIVITY
c       SLOPE : SLOPE OF TEMPERATURE VS. F(ALPHA,TAU)
c       ICOND : NUMBER OF CONDUCTIVITY ITERATIONS
c       ISHIFT : TOTAL NUMBER OF ITERATIONS
c
c***********************************************************************
c
       subroutine cond(i,numb,time,temp,axe,est,nused,
     $                  ftau,tem,k,rms,slope,icond,ishift)
c
       implicit real (k)
       common/sensor/nsensr,tcycle,deltat,a,dsensr,terror,plen,
     $                aa(15),bb(15),cc(15),
     $                ratcl(3),f(5000),alpha,fstart,fincr
       common/condco/kinit(15),ktoler,tsinit,tsincr,tpmin,tpmax,
     $                ipmax,pdelay(15),power
       common/out/lundat,lunout,long,lplot
c 
       dimension tau(75),ftau(75),tem(75),err(75),temp(75),
     $            time(75),timinv(75),temc(75),kpoint(75)
c
       nused = 0
       pi=4.*atan(1.)
       kappa = ratbul(kinit(i))
       if(est.eq.0.)   est = terror
c
c.......SELECTION OF DATA POINTS IN THE PRESET TIME-WINDOW
c       NEGATIVE TEMPERATURES ARE EXCLUDED
c
c       THE USER OF THE ALGORITHM SHOULD BE AWARE OF THE FACT
c       THAT IN THE IDEAL CASE WITH PERFECT DATA THE DATA SET 
c       SHOULD BE DECIMATED AT THIS POINT TO EQUALLY SPACED 
c       POINTS IN THE 1/TIME-SPACE. FROM OUR EXPERIENCE WITH 
c       DIFFERENT HEAT FLOW INSTRUMENTS AND THEIR CHARACTERISTIC
c       NOISE PROBLEMS, HOWEVER, INCLUSION OF ADDITIONAL POINTS MAY BE
c       STATISTICALLY WARRENTED.
c
c
       do       100       j=1,numb
       tau(j) = (kappa * time(j))/( a*a )
       if(tau(j).ge.tpmin.
     $    and.tau(j).le.tpmax.
     $        and.tau(j).ge.fstart.
     $            and.temp(j).gt.0.0)     then
           nused = nused + 1
           time(nused)=time(j)
           tau(nused) = tau(j)
           tem(nused) = temp(j)
           err(nused) = est
       endif
100    continue
c
c.......
c
       if(long.ne.0)   then
         write(lunout,*)
         write(lunout,*)
         write(lunout,*)'       sensor : ',i
         write(lunout,*)'tau-falphatau-temperature,npused : ',nused
         do       200       l=1,nused,6
         write(lunout,30)(tau(l+j-1),tem(l+j-1),j=1,6)
30       format(1h ,6(f5.3,1x,f5.3,'*'))
200      continue
         write(lunout,*)'icond,ishift,ts,k,kslope,axe,rms,cor'
       endif
c
       ts = pdelay(i)
       k = kinit(i)
c
       ishift = 0
       icond = 0
c      
       if(nused.eq.0)    return
c
c.......MAIN LOOP FOR THE ITERATIVE DETERMINATION OF THE CONDUCTIVITY
c       AND THE EFFECTIVE TIME ORIGIN
c
       do       300       j=1,ipmax
c
       icond = icond + 1
       kappa = ratbul(k)
c
              do 400   l=1,ipmax
c
              ishift = ishift + 1
              sum = 0.
c
c.......CALCULATION OF KPOINT USING TEMPERATURES
c
                     do       500       m=1,nused

                     dtime = time(m) + ts
                           if(dtime.lt.1.)  then
                              icond = 99
                              ishift = 99
                           return
                     endif

                     timinv(m) = 1./dtime
                     tau(m) = (kappa*dtime)/(a*a)
                     ftau(m) = ftable(tau(m))
                     temc(m) = tem(m)/(2.*alpha*tau(m)*ftau(m))
                     kpoint(m) = (power*timinv(m))/(4.*pi*temc(m))
                     sum = sum + kpoint(m)

500                  continue
c
              call least(timinv,temc,err,nused,axe,slope,vara,varb,
     $                covar,cor,rms)
c
c.......CALCULATION OF KSLOPE USING THE SLOPE
c
              kslope=(power)/(4.*pi*slope)
              dk=kslope-(power)/(4.*pi*(slope+vara))
              kper=(dk/kslope)*100.
              kdif=k - kslope
c
c.......IF KDIF IS GREATER THEN KTOLER TS HAS TO BE ADJUSTED ITERATIVELY
c
             if(abs(kdif).ge.ktoler)   then
             if(long.ne.0)    then
             write(lunout,10)icond,ishift,ts,k,kslope,axe,rms,cor
10           format(1h ,2i4,f8.2,2f8.3,f8.3,e12.4,f8.4)
             endif
             else
             if(long.ne.0)    then
             write(lunout,10)icond,ishift,ts,k,kslope,axe,rms,cor
             endif
             goto  1000
             endif
c
c.......START VALUES FOR THE REGULA FALSI (FALSE POSITION)
c
              if(l.eq.1)       then
              start=kdif
              c=ts
              ts=ts+tsincr
              goto       400
              else
c
c.......CALCULATION OF THE TIME STEP
c
              zael=c*kdif-ts*start
              ts=zael/(kdif-start)
              endif
c
400              continue
c
1000       continue
c
       call fmvsd(kpoint,nused,kmean,kvar,ksigma)
c
       if(abs(k-kmean).le.ktoler)       goto 2000
c
c.......START VALUES FOR THE REGULA FALSI CONDUCTIVITY ITERATION
c
       if(j.eq.1)       then
       startf = axe
       cd = kmean
       k = kmean
       goto       300
       else
c
c.......CALCULATION OF THE CONDUCTIVITY STEP SIZE
c
       zael1 = cd*axe - kmean*startf
       k = zael1 / (axe-startf)
       endif
c
300       continue
c
c.......THIS POINT OF THE PROGRAM IS ONLY REACHED IF
c       K(ASSUMED)=K(SLOPE)=K(POINT)
c
2000       continue
c
       if(long.ne.0)   then
       write(lunout,*)'final values for sensor : ',i
       write(lunout,*)'icond,ishift,ts,k,kslope,kmean,ksigma,axe,rms,co
     $                 r'
       write(lunout,20)icond,ishift,ts,k,kslope,kmean,ksigma,
     $                  axe,rms,cor
20     format(1h ,2i4,f7.2,4f8.4,f8.3,e12.4,f8.4)
       endif
c
c
c.......FINAL CALCULATION OF THE TEMPERATURE VS. F(ALPHA,TAU)
c       RELATION
c
       k = (kmean+kslope+k)/3.
       pdelay(i) = ts
       kappa = ratbul(kmean)
       do       600       j=1,nused
       tau(j)=((time(j)+ts)*kappa) / (a*a)
       ftau(j) = ftable(tau(j))
600       continue
c
       call least(ftau,tem,err,nused,axe,slope,vara,varb,
     $             covar,cor,rms)
c
       return
       end











