C***********************************************************************
C
C
C
C       AUTHOR : H. VILLINGER / E.E. Davis
C
C
C       TITLE :  HFRED
C
C
C
C       DATE : 1984/1985
C
C
C
C
C       PURPOSE
C       -------
C
C       REDUCTION OF HEAT FLOW PENETRATION MEASUREMENTS MADE WITH A
C       VIOLIN-BOW TYPE HEAT PROBE AND IN SITU THERMAL CONDUCTIVITY
C       MEASUREMENTS USING THE PULSE PROBE METHOD.
C
C
C       ALGORITHM
C       ---------
C       THE MAIN PURPOSE OF THE PROGRAM IS TO DETERMINE ITERATIVELY
C       THE EFFECTIVE ORIGIN TIMES FOR THE FRICTIONAL AND THE HEAT
C       PULSE DECAY. THE FINAL HEAT FLOW VALUE IS  N O T  CALCULATED
C       BUT HAS TO BE DETERMINED MANUALLY.
C       THE PROGRAM ITSELF IS NOT EQUIPPED WITH A LOT OF ERROR
C       TRAPPING ROUTINES SO THE USER SHOULD MAKE SURE IN ADVANCE
C       THAT FOR EXAMPLE THE DATA FILE IS SET UP PROPERLY.
C       THE ALGORITHM OF THE PROGRAM IS DESCRIBED IN DETAIL IN :
C
C       VILLINGER, H. AND DAVIS, E.E.  (1987)
C       A NEW REDUCTION ALGORITHM FOR MARINE HEAT-FLOW MEASUREMENTS.
C       JOURNAL OF GEOPHYSICAL RESEARCH (IN PRESS)
C                         
C       THE READING OF THE FOLLOWING PAPERS IS HIGHLY SUGGESTED
C       BEFORE USING THE PROGRAM :
C
C       HYNDMAN,R.D.,DAVIS,E.E. AND WRIGHT,J.A.(1979)
C       THE MEASUREMENT OF MARINE GEOTHERMAL HEAT FLOW BY A MULTI-
C       PENETRATION PROBE WITH DIGITAL ACOUSTIC TELEMETRY AND IN
C       SITU THERMAL CONDUCTIVITY.
C       MARINE GEOPHYSICAL RESEARCHES,4,181-205
C
C       C.R.B.LISTER (1979)
C       THE PULSE-PROBE METHOD OF CONDUCTIVITY MEASUREMENT.
C       GEOPHYS.J.R.ASTR.SOC.,57,451-461
C
C       DAVIS,E.E.,LISTER,C.R.B. AND SCLATER,J.G. (1984)
C       TOWARDS DETERMINING THE THERMAL STATE OF OLD OCEAN LITHO-
C       SPERE : HEAT FLOW MEASUREMENTS FROM THE BLAKE-BAHAMA OUTER
C       RIDGE,NORTH-WESTERN ATLANTIC.
C       GEOPHYS.J.R.ASTR.SOC.,78,507-545
C
C
C
C
C       FILE USAGE  (ONLY FOR THIS PROGRAM SEGMENT)
C       ------------------------------------------
C
C       2  = PARAMETER - FILE (INPUT)
C       11 = RESULT DISK FILE (OUTPUT)
C       *  = KEYBOARD/SCREEN
C
C
C       EXPLANATION OF SUBROUTINES USED
C       -------------------------------
C       GETPAR : LOADS THE PARAMETERS FROM A PARAMETER FILE HFRED.PAR
C       PARAM : PRINTS THE PARAMETERS OF HFRED.PAR
C       TABLE : LOADS THE F(ALPHA,TAU)-TABLE USED
C               THE TABLE HAS TO BE GENERATED BY THE USER USING THE 
C               PROGRAM FALTAU LISTED ALSO IN THIS REPORT
C       TITLE : READS THE HEADER INFORMATION FROM THE
C               PENETRATION DATA FILE
C       CHANGE : CHANGES PRESET PARAMETERS READ IN FROM
C                HFRED.PAR
C       READ : READS THE PENETRATION DATA FROM DISK ,CONVERTS THE DATA
C              AND SPLITS THE DATA SET INTO FRICTIONAL DECAY AND HEAT
C              PULSE DECAY.
C       FRIC : CALCULATES INFINITE TIME SEDIMENT TEMPERATURES
C       FOUT : PRINTS THE RESULTS FROM FRIC
C        COR : CORRECTS THE HEAT PULSE DECAY TEMPERATURES FOR RESIDUAL
C              FRICTIONAL HEATING
C       COND : CALCULATES THE IN SITU THERMAL CONDUCTIVITY
C       COUT : PRINTS RESULTS FROM COND
C       PLOTID : ASSIGNS A PLOT-ID NUMBER FOR THE PLOT
C       FINRES : PRINTS THE FINAL RESULTS FROM FRIC AND COND
C       RESULT : PLOTS THE FRICTIONAL AND HEAT PULSE DECAY, TEMPERATURE
C                AND THERMAL CONDUCTIVITY VS. DEPTH AND THERMAL
C                RESISTANCE VS. DEPTH
C       ZERO : SETS ALL ELEMENTS IN THE MAIN ARRAYS TO ZERO
C       FMVSD : COMPUTES THE MEAN, VARIANCE AND STANDART DEVIATION
C               OF AN ARRAY (FILE NAME IS MEAN)
C       BDWT : CALCULATES THERMAL RESISTANCES (AUTHORS : MARK WIEDER-
C              SPAN/U.TEXAS AT AUSTIN AND CLIVE LISTER/U.WASHINGTON
C              SEATTLE)
C       RATBUL : CALCULATES A THERMAL DIFFUSIVITY FROM THE THERMAL
C                CONDUCTIVITY USING THE RELATIONSHIP DESCRIBED
C                IN HYNDMAN ET AL. 1979 (SEE ABOVE)
C       TEMP : CONVERTS READINGS INTO TEMPERATURES
C       LEAST : CALCULATES A LEAST SQUARE FIT
C       FTABLE : INTERPOLATES FROM THE F(ALPHA,TAU) TABLE USED
C
C
C       VARIABLES
C       ---------
C
C       A DETAILED VARIABLE DESCRIPTION WILL BE GIVEN IN THE SUB-
C       ROUTINES WHERE THEY ARE INTRODUCED. ONLY THE VARIABLES
C       USED IN COMMON BLOCKS WILL BE DESCRIBED HERE.
C
C
C       COMMON BLOCK DECDAT  ( DECAY DATA )
C
C       TFRIC(J,K) : TIME OF FRICTIONAL DECAY DATA POINT J FOR SENSOR K
C       RFRIC(J,K) : VALUE OF FRICTIONAL DACAY DATA POINT J FOR SENSOR K
C       TPULS(J,K) : TIME OF HEAT PULSE DECAY DATA POINT J FOR SENSOR K
C       RPULS(J,K) : VALUE OF HEAT PULSE DECAY DATA POINT J FOR SENSOR K
C
C
C       COMMON BLOCK PDATA (PLOT DATA)
C
C       TFPLOT(J,K) : TIME OF FRICTIONAL DECAY DATA POINT J FOR SENSOR K
C       RFPLOT(J,K) : VALUE OF FRICTIONAL DACAY DATA POINT J FOR SENSOR K
C       TPPLOT(J,K) : TIME OF HEAT PULSE DECAY DATA POINT J FOR SENSOR K
C       RPPLOT(J,K) : VALUE OF HEAT PULSE DECAY DATA POINT J FOR SENSOR K
C                     
C
C
C       COMMON BLOCK SENSOR (SENSOR DATA)
C
C       NSENSR : NUMBER OF SENSORS (MAX. 15)
C       TCYCLE : TIME BETWEEN MEASURMENT CYCLES [SEC]
C       DELTAT : TIME BETWEEN MEASUREMENTS AT CONSECUTIVE SENSORS [SEC]
C       A      : RADIUS OF THE SENSOR STRING  [M]
C       DSENSR : LENGTH INCREMENT BETWEEN SENSORS [M]
C       TERROR : ASSUMED ERROR IN THE TEMPERATURE MEASUREMENTS [K]
C       PLEN   : LENGTH OF THE POWER INPUT [SEC]
C       AA,BB,CC : CALIBRATION COEFFICIENTS FOR THE CONVERSION OF
C                  READINGS INTO TEMPERATURES ASSUMING A POLYNOMIAL
C                  OF THIRD DEGREE (SEE FUNCTION TEMP)
C                  THE COEFFICIENTS HAVE TO BE SPECIFIED FOR EACH SENSOR
C       RATCL : COEFFICIENTS FOR THE RELATIONSHIP BETWEEN THE THERMAL
C               CONDUCTIVITY AND THERMAL DIFFUSIVITY
C       F : DATA OF THE F(ALPHA,TAU)-TABLE USED
C       FSTART : START VALUE OF THE F(ALPHA,TAU) TABLE
C       FINCR : INCREMENT OF TAU IN THE USED F(ALPHA,TAU) TABLE
C
C
C       COMMON BLOCK FRICO
C
C       FDELAY(J) : TIME DELAY FOR FRICTIONAL DELAY FOR SENSOR J [SEC]
C       DINCR : TIME INCREMENT USED IN THE FDELAY ITERATION [SEC]
C       IFMAX : MAX. NUMBER OF STEPS
C       TFMIN/TFMAX : MIN. AND MAX. TAU VALUES USED FOR FRICTIONAL DELAY
C
C
C       COMMON BLOCK CONDCO
C
C       KINIT(J) : INITIAL THERMAL CONDUCTIVITY [W/MK] FOR SENSOR J
C       KTOLER : ERROR BOUNDS FOR CONDUCTIVITY ITERATION [W/MK]
C       TSINIT : INITIAL TIME SHIFT [SEC]
C       TSINCR : INITIAL TIME SHIFT INCREMENT [SEC]
C       TPMIN/TPMAX : MIN. AND MAX. TAU VALUES USED FOR THE THERMAL
C                     CONDUCTIVITY DETERMINATION
C       IPMAX : MAX. ITERATION NUMBER
C       PDELAY(J) : TIME DELAY FOR HEAT PULSE DECAY FOR SENSOR J [SEC]
C       POWER : ENERGY INPUT PER LENGTH [JOULE/M]
C
C
C       COMMON BLOCK OUT
C
C       LUNDAT : UNIT NUMBER FOR DATA FILE
C       LUNOUT : UNIT NUMBER FOR OUTPUT FILE
C       LONG : IF LONG=1 THEN AN EXTENSIVE PRINTOUT IS CREATED WHICH IS HELPFUL
               IN CASES WHERE HFRED DOES NOT CONVERGE.
C       LPLOT : -- NOT USED AT THE MOMENT --
C
C
C       COMMON BLOCK PLOTPA (PLOT PARAMETER)
C
C       FOR DETAILS SEE SUBROUTINE RESULT
C
C***********************************************************************
c
       program hfred
c
       implicit real(k)
       character answ*1,fname*8,cruise*10
        dimension nfric(15),npuls(15),rwater(15),
     $            sedtem(15),estsed(15),sedrms(15),fslope(15),nfused(15),
     $            k(15),ktem(15),pslope(15),krms(15),npused(15),
     $            icond(15),ishift(15),
     $            work1(75),work2(75),time(75),tem(75)
       common/decdat/tfric(75,15),rfric(75,15),
     $               tpuls(75,15),rpuls(75,15)
       common/pdata/tfplot(75,15),rfplot(75,15),
     $              tpplot(75,15),rpplot(75,15)
       common/sensor/nsensr,tcycle,deltat,a,dsensr,terror,plen,
     $               aa(15),bb(15),cc(15),
     $               ratcl(3),f(5000),alpha,fstart,fincr
       common/frico/fdelay(15),dincr,ifmax,tfmin,tfmax
       common/condco/kinit(15),ktoler,tsinit,tsincr,tpmin,tpmax,
     $               ipmax,pdelay(15),power
       common/out/lundat,lunout,long,lplot
       common/plotpa/fp(8),tp(8),cp(8),hp(8),bp(8)
c
c.......
c
       write(*,*)'              HEAT FLOW REDUCTION PROGRAM '
       write(*,*)'              ==========================='
       write(*,*)' '
       write(*,*)'          Pacific Geoscience Centre  October 1984'
c
c.......GETPAR : LOADS THE PARAMETERS FROM THE PARAMETER FILE
c                HFRED.PAR
c
       call getpar(2)
c
c.......TABLE : LOADS THE F(ALPHA,TAU)-TABLE USED
c
       call table
c
c.......TITLE : READS THE HEADER INFORMATION FROM THE PENETRATION
c               DATA FILE AND PRINTS IT
c
2000       call title(istn,ipen,cruise)
c
c.......PARAM : PRINTS THE PARAMETERS OF HFRED.PAR
c
       call param
c
c.......CHANGE : CHANGES PRESET PARAMETERS READ IN FROM
c                HFRED.PAR
c
       write(*,*)'DO YOU WANT TO CHANGE PARAMETERS  Y / N '
       read(*,10)answ
       if(answ.eq.'Y'.or.answ.eq.'y')   then
         call change
         call param
       endif
c
c.......READ : READS THE PENETRATION DATA FROM DISK ,CONVERTS THE DATA
c              AND SPLITS THE DATA SET INTO FRICTIONAL DECAY AND HEAT
c              PULSE DECAY.
c
            call read(nfric,npuls,rwater,fricon,pulson)
c
c.......FRIC : CALCULATES INFINITE TIME SEDIMENT TEMPERATURES
c
          msum = 0
1000              do       100       i=1,nsensr
                      if(nfric(i).eq.0)   goto 100
                 kappa=ratbul(kinit(i))
                 call fric(i,nfric(i),kappa,work1,work2,
     $                sedtem(i),estsed(i),sedrms(i),fslope(i),nfused(i))
                  msum = msum + nfused(i)
c
                            do       200       j=1,nfused(i)
                                    tfplot(j,i)=work1(j)
                                     rfplot(j,i)=work2(j)
                                     work1(j) = 0.
                                    work2(j) = 0.
200                            continue
c
100              continue
c
c.......FOUT : PRINTS THE RESULTS FROM FRIC
c
       if(msum.gt.0)  then
             call fout(nfric,nfused,sedtem,estsed,sedrms,fslope,rwater)
       endif
c
c.......
c
              msum = 0
              do       300       i=1,nsensr
c
                     if(npuls(i).eq.0)       then
                              k(i) = kinit(i)
                              goto       300
                     endif
c
c.......COR : CORRECTS THE HEAT PULSE DECAY TEMPERATURES FOR RESIDUAL
c             FRICTIONAL HEATING
c
              call cor(i,time,tem,npuls,sedtem,fslope,fricon,pulson)
c
c.......COND : CALCULATES THE IN SITU THERMAL CONDUCTIVITY
c
              call cond(i,npuls(i),time,tem,ktem(i),estsed(i),
     $                   npused(i),work1,work2,k(i),krms(i),
     $                   pslope(i),icond(i),ishift(i))
c
c.......
c
                  msum = npused(i) + msum
                     do       400       j=1,npused(i)
                     tpplot(j,i) = work1(j)
                     rpplot(j,i) = work2(j)
                     work1(j) = 0.
                     work2(j) = 0.
400                  continue
c
300              continue
c
c.......COUT : PRINTS RESULTS FROM COND
c
       if(msum.gt.0)  then
              call cout(npuls,npused,ktem,k,krms,pslope,icond,ishift)
       endif
c
c.......PLOTID : ASSIGNS A PLOT-ID NUMBER FOR THE PLOT
c       FINRES : PRINTS THE FINAL RESULTS FROM FRIC AND COND
c       RESULT : PLOTS THE FRICTIONAL AND HEAT PULSE DECAY, TEMPERATURE
c                AND THERMAL CONDUCTIVITY VS. DEPTH AND THERMAL
c                    RESISTANCE VS. DEPTH
c
4000       write(*,*)' DO YOU WANT TO PLOT THE DATA ?   Y/N'
           read(*,10)answ 
              if(answ.eq.'Y'.or.answ.eq.'y')   then
                 call plotid(pid)
             call finres(istn,ipen,sedtem,k,pid,weit,npused)
             call result(nfused,sedtem,fslope,istn,ipen,
     $                    npused,k,ktem,pslope,pid,weit)
                   else
             pid=0.
              call finres(istn,ipen,sedtem,k,pid,weit,npused)
              endif
c
c.......
c
         write(*,*)' SAVE THE RESULTS ON DISKETTE ? Y/N'
         read(*,10)answ
          if(answ.eq.'Y'.or.answ.eq.'y')   then
             idnt = pid
             write(fname,90)istn,ipen,idnt  
             open(11,file=fname,status='new')
             write(11,110)istn,ipen,idnt,cruise
             do		800 l=1,nsensr
c
             write(11,120)l,sedtem(l),estsed(l),fdelay(l),fslope(l),
     $                     k(l),krms(l),pdelay(l),pslope(l)
800          continue
             close(11)
          endif
c
c.......
c
            write(*,*)' ANOTHER ITERATION ?   Y/N'
       read(*,10)answ
       if(answ.eq.'Y'.or.answ.eq.'y')   then
           write(lunout,20)
                do           500             i=1,nsensr
          fdelay(i) = abs(fdelay(i)/2.)
          kinit(i) = k(i)
           pdelay(i) = pdelay(i)-(pdelay(i)*0.3)
500          continue
c
               write(*,*)' DO YOU WANT TO CHANGE PARAMETERS?   Y/N'
              read(*,10)answ
          if(answ.eq.'Y'.or.answ.eq.'y')      call change
c
          call param
          goto 1000
       endif
c
       write(*,*)' ANOTHER PENETRATION ?   Y/N'
       read(*,10)answ
       if(answ.eq.'Y'.or.answ.eq.'y')  then
          write(lunout,20)
c
c.......ZERO : SETS ALL ELEMENTS IN THE MAIN ARRAYS TO ZERO
c
           do       600       i=1,nsensr
              call zero(nfric,npuls,rwater,
     $                  sedtem,estsed,sedrms,
     $                           fslope,nfused,
     $                  k,ktem,pslope,krms,npused,
     $                  icond,ishift,
     $                  work1,work2,time,tem)


600          continue
c
             call getpar(2)
c
              goto 2000
c
       endif
c
10     format(a1)
20     format(1h1)
90     format(i2.2,i3.3,i3.3)
110    format(3i5,1x,1h',a10,1h')
120    format(i2,1x,f8.3,1x,f8.4,1x,f8.2,1x,f8.3,1x,f8.3,1x,f8.4,1x,
     $        f8.2,1x,f8.3)
       stop
       end





